#include <iostream>
#include <boost/program_options.hpp>
#include <string>
#include <vector>
#include "tbb/task_scheduler_init.h"

#include "configuration.hh"
#include "pipeline.hh"

#include "encoder/video_reader.hh"
#include "encoder/video_writer.hh"

bool parse_cli(Configuration& conf, int argc, char** argv)
{
  namespace po = boost::program_options;

  // Register all possible options
  po::options_description desc("Allowed options");
  desc.add_options()
    ("help,h", "produce help message")
    ("filter,f", po::value<std::vector<std::string>>(), "apply a filter")
    ("input-file,i", po::value<std::vector<std::string>>(), "file to treat")
    ("output,o", po::value<std::string>(), "output file")
    ("codec,c", po::value<std::string>(), "output codec")
    ;

  // Handle ./prpa file.avi instead of ./prpa --input-file=file.avi
  po::positional_options_description p;
  p.add("input-file", -1);

  // Read command line
  po::variables_map vm;
  try
  {
    po::store(po::command_line_parser(argc, argv).
              options(desc).positional(p).run(), vm);
  }
  catch (po::invalid_command_line_syntax e)
  {
    std::cerr << argv[0] << ": " << e.what() << std::endl;
    return false;
  }
  po::notify(vm);

  // Handle -h and --help
  if (argc <= 1 || vm.count("help"))
  {
    std::cout << desc << "\n";
    return false;
  }

  // GO!
  std::cout << "Given filters: ";
  if (vm.count("filter"))
    std::cout << vm["filter"].as<std::vector<std::string> > ().size()
              << std::endl;
  else
    std::cout << 0 << std::endl;

  std::cout << "Given files: ";
  if (vm.count("input-file"))
    std::cout << vm["input-file"].as<std::vector<std::string> > ().size()
              << std::endl;
  else
    std::cout << 0 << std::endl;

  /* Store configuration */
  conf.output = std::string(vm["output"].as<std::string>());
  conf.codec = std::string(vm["codec"].as<std::string>());

  std::vector<std::string> vs;
  if (vm.count("input-file"))
  {
    vs = vm["input-file"].as<std::vector<std::string>>();
    for (auto it = vs.begin(); it != vs.end(); ++it)
      conf.input.push_back(std::string(*it));
  }

  if (vm.count("filter"))
  {
    vs = vm["filter"].as<std::vector<std::string>> ();
    for (auto it = vs.begin(); it != vs.end(); ++it)
      conf.filters.push_back(std::string(*it));
  }

  return true;
}

int main (int argc, char** argv)
{
  tbb::task_scheduler_init init;
  Configuration conf = Configuration();

  if (!parse_cli(conf, argc, argv))
    return 1;

  VideoReader vr(conf.input.front());
  auto it = conf.input.begin();
  it++;
  VideoWriter vw = VideoWriter();
  vw.open(conf.output, conf.codec, vr.fps_get(), vr.width_get(), vr.height_get());

  Pipeline pipe(std::list<VideoReader> { vr, VideoReader(*it) }, vw);
  // add filters here
  //pipe.add_filter("grey");
  //pipe.add_filter("reverse");
  //pipe.add_filter("interlace");
  //pipe.add_filter("merge");
  pipe.run();

  return 0;
}
